#include "StdAfx.h"
#include ".\connection.h"

#include <curl/curl.h>
#include <curl/types.h>
#include <curl/easy.h>

/********************************************************
	// Constructors & Destructors
********************************************************/
CConnection::CConnection(HTTP_Mode Mode, char* pHost, char* pFilePath, char* pArg, unsigned int Port, bool binary)
{
	// Initialize cURLpp
	cURLpp::initialize();

	// allocate the necessary memory
	this->m_pHostAdress = (char*) malloc((strlen(pHost) + 1) * sizeof(char));
	this->m_pFilePath = (char*) malloc((strlen(pFilePath) + 1) * sizeof(char));
	this->m_pArg = (char*) malloc((strlen(pArg) + 1) * sizeof(char));

	// Set host adress & port number
	strcpy(this->m_pHostAdress, pHost);
	strcpy(this->m_pFilePath, pFilePath);
	strcpy(this->m_pArg, pArg);
	this->m_PortNumber = Port;

	// Create the request object
	this->m_pRequest = new cURLpp::Easy();

	// Create the memory buffer
	this->m_pWriterChunk = new CConnection::WriterMemoryClass();
	this->m_pReaderChunk = new CConnection::ReaderMemoryClass();
	this->m_pDebugChunk = new CConnection::DebugMemoryClass();

	// Initialize the request object
	this->Init(pHost, pFilePath, pArg, Port, Mode);
}

CConnection::~CConnection(void)
{
	// Free allocated memory
	if (this->m_pHostAdress)
	{
		free(this->m_pHostAdress);
	}
	if (this->m_pFilePath)
	{
		free(this->m_pFilePath);
	}
	if (this->m_pArg)
	{
		free(this->m_pArg);
	}

	// Terminate the cURLpp
	cURLpp::terminate();
}

/********************************************************
	// Initialisation
********************************************************/
void CConnection::Init(char* pHost, char* pFilePath, char* pArg, unsigned int Port, HTTP_Mode Mode)
{
	try
	{
		// Set the timeout
		{
			// The maximum time  in seconds that you allow the libcurl transfer operation to take.
			// Normally, name lookups  can  take  a  considerable time  and limiting operations to 
			// less than a few minutes risk aborting perfectly normal operations.
			// This  option will cause curl to use the SIGALRM to enable time-outing system calls.
			//
			// NOTE: this does not work  in  Unix  multi-threaded  programs, as it uses signals.
			this->m_pRequest->setOpt(new cURLpp::Options::Timeout(120));			// TIMEOUT set to 2 min...

			// The maximum time in  seconds  that  you  allow  the  connection to the server to take.
			// This only limits the connection  phase,  once  it has  connected,  this  option  is of no more use.
			// Set to zero to disable connection timeout (it  will  then  only timeout on the system's internal timeouts).
			this->m_pRequest->setOpt(new cURLpp::Options::ConnectTimeout(30));		// <- CONECTIONTIMEOUT set to 30 sec...
		}

		// Creation of the URL option.
		this->m_pRequest->setOpt(new cURLpp::Options::Url(this->m_pHostAdress));

		// Creation of the port option
		// this->m_pRequest->setOpt(new cURLpp::Options::Port(this->m_PortNumber));

		// Set verbose mode
		this->m_pRequest->setOpt(new cURLpp::Options::Verbose(true));

		// Set user-agent field
		this->m_pRequest->setOpt(new cURLpp::Options::UserAgent("libcurl-agent/1.0"));

		// Set the writer callback to enable cURL to write result in a memory area
		this->m_pRequest->setOpt(new cURLpp::Options::WriteFunction(CConnection::WriterMemoryClass::WriteMemoryCallback));

		// The class (WriterMemoryClass) passed to out callback function which holds our data received from the server
		this->m_pRequest->setOpt(new cURLpp::Options::WriteData((void*) (this->m_pWriterChunk)));

		// Set the writer callback to enable cURL to write result in a memory area
		this->m_pRequest->setOpt(new cURLpp::Options::DebugFunction(CConnection::DebugMemoryClass::DebugMemoryCallback));

		// The class (DebugMemoryClass) passed to out callback function which holds our debug data
		this->m_pRequest->setOpt(new cURLpp::Options::DebugData((void*) (this->m_pDebugChunk)));

		// Check if we have HTTP_POST mode set
		if (Mode == CConnection::HTTP_Mode::HTTP_POST)
		{
			std::string SOAPString;
			std::list<std::string> HeaderData;

			// Construct the final SOAP envelope to send to the server...
			SOAPString.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			SOAPString.append("<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" \
								xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" \
								xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">");
			SOAPString.append("<soapenv:Body>");
			SOAPString.append(pArg);				// <- The AWQL!
			SOAPString.append("</soapenv:Body>");
			SOAPString.append("</soapenv:Envelope>");

			// Set the String to uploaded in the ReaderMemoryClass -> Uploading must be enabled!!!
			this->m_pReaderChunk->SetBuffer((char*) SOAPString.c_str());

			// Activate the POST mode
			this->m_pRequest->setOpt(new cURLpp::Options::Post(true));

			// Set the reading callback & reading data to send to host
			this->m_pRequest->setOpt(new cURLpp::Options::ReadFunction(CConnection::ReaderMemoryClass::ReadMemoryCallback));
			this->m_pRequest->setOpt(new cURLpp::Options::ReadData((void*) (this->m_pReaderChunk)));
		}
		// Output the options set
		// MessageBox(NULL, this->m_pRequest->print(), NULL, NULL);
	}
	catch (cURLpp::RuntimeError &e)
	{
		std::cout << e.what() << std::endl;
		MessageBox(NULL, e.what(), NULL, NULL);
	}
	catch (cURLpp::LogicError &e)
	{
		std::cout << e.what() << std::endl;
		MessageBox(NULL, e.what(), NULL, NULL);
	}
	catch (...)
	{
		std::cout << "Undefined cURLpp Exception!!!" << std::endl;
		MessageBox(NULL, "Undefined cURLpp Exception!!!", NULL, NULL);
	}
}

/********************************************************
	// Getter & Setter Methods
********************************************************/
void CConnection::SetHostAdress(char* pHost)
{
	// "Method://Machine(:Port)/Path/Document"
	//
	// Copy the string to local member var
	if (this->m_pHostAdress)
	{
		free(this->m_pHostAdress);
	}
	this->m_pHostAdress = (char*) malloc((strlen(pHost) + 1) * sizeof(char));
	strcpy(this->m_pHostAdress, pHost);

	try
	{
		this->m_pRequest->setOpt(new cURLpp::Options::Url(this->m_pHostAdress));
	}
	catch (cURLpp::RuntimeError &e)
	{
		std::cout << e.what() << std::endl;
		MessageBox(NULL, e.what(), NULL, NULL);
	}
	catch (cURLpp::LogicError &e)
	{
		std::cout << e.what() << std::endl;
		MessageBox(NULL, e.what(), NULL, NULL);
	}
	catch (...)
	{
		std::cout << "Undefined cURLpp Exception!!!" << std::endl;
		MessageBox(NULL, "Undefined cURLpp Exception!!!", NULL, NULL);
	}
}

void CConnection::SetFilePath(char* pPath)
{
	// Copy the string to local member var
	if (this->m_pFilePath)
	{
		free(this->m_pFilePath);
	}
	this->m_pFilePath = (char*) malloc((strlen(pPath) + 1) * sizeof(char));
	strcpy(this->m_pFilePath, pPath);
}

void CConnection::SetArg(char* pArg)
{
	// Local vars...
	std::string SOAPString;

	// Copy the string to local member var
	if (this->m_pArg)
	{
		free(this->m_pArg);
	}
	this->m_pArg = (char*) malloc((strlen(pArg) + 1) * sizeof(char));
	strcpy(this->m_pArg, pArg);

	std::list<std::string> HeaderData;
	char ContLength[50];

	HeaderData.push_back("SOAPAction:  \r");
	HeaderData.push_back("Accept: text/html, image/gif, image/jpeg, *; q=.2, */*; q=.2\r");
	HeaderData.push_back("Connection: keep-alive\r");
	HeaderData.push_back("Content-Type: application/x-www-form-urlencoded\r");

	// Construct the final SOAP envelope to send to the server...
	SOAPString.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
	SOAPString.append("<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" \
						xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" \
						xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">");
	SOAPString.append("<soapenv:Body>");
	SOAPString.append(this->m_pArg);				// <- The AWQL!
	SOAPString.append("</soapenv:Body>");
	SOAPString.append("</soapenv:Envelope>");

	sprintf(ContLength, "Content-Length: %d\r", strlen(SOAPString.c_str()));
	HeaderData.push_back(ContLength);
	HeaderData.push_back("\r");
	HeaderData.push_back("\r");

	// Set the String to uploaded in the ReaderMemoryClass -> Uploading must be enabled!!!
	this->m_pReaderChunk->SetBuffer((char*) SOAPString.c_str());

	try
	{
		this->m_pRequest->setOpt(new cURLpp::Options::HttpHeader(HeaderData));
		this->m_pRequest->setOpt(new cURLpp::Options::HeaderData((void*) &HeaderData));
		// Set the reading callback & reading data to send to host
		this->m_pRequest->setOpt(new cURLpp::Options::InfileSize(strlen(this->m_pReaderChunk->GetBuffer())));
	}
	catch (cURLpp::RuntimeError &e)
	{
		std::cout << e.what() << std::endl;
		MessageBox(NULL, e.what(), NULL, NULL);
	}
	catch (cURLpp::LogicError &e)
	{
		std::cout << e.what() << std::endl;
		MessageBox(NULL, e.what(), NULL, NULL);
	}
	catch (...)
	{
		std::cout << "Undefined cURLpp Exception!!!" << std::endl;
		MessageBox(NULL, "Undefined cURLpp Exception!!!", NULL, NULL);
	}
}

void CConnection::SetPortNumber(unsigned int PortNumber)
{
	// Set the port number to local member var
	this->m_PortNumber = PortNumber;

	try
	{
		this->m_pRequest->setOpt(new cURLpp::Options::Port(this->m_PortNumber));
	}
	catch (cURLpp::RuntimeError &e)
	{
		std::cout << e.what() << std::endl;
		MessageBox(NULL, e.what(), NULL, NULL);
	}
	catch (cURLpp::LogicError &e)
	{
		std::cout << e.what() << std::endl;
		MessageBox(NULL, e.what(), NULL, NULL);
	}
	catch (...)
	{
		std::cout << "Undefined cURLpp Exception!!!" << std::endl;
		MessageBox(NULL, "Undefined cURLpp Exception!!!", NULL, NULL);
	}
}

void CConnection::SetLoggingHWND(HWND hWnd)
{
	this->m_pDebugChunk->SetLoggingHWND(hWnd);
}

char* CConnection::GetHostAdress(void)
{
	return this->m_pHostAdress;
}

unsigned int CConnection::GetPortNumber(void)
{
	return this->m_PortNumber;
}

char* CConnection::GetResult(void)
{
	return this->m_pWriterChunk->GetBuffer();
}

int CConnection::GetResultLength(void)
{
	return this->m_pWriterChunk->GetSize();
}

/********************************************************
	// Helper Methods
********************************************************/
void CConnection::PerformAction(void)
{
	try
	{
		// Perform the transaction with the options set.
		this->m_pRequest->perform();

		// Print all the option values...
		// this->m_pRequest->print();
	}
	catch (cURLpp::RuntimeError &e)
	{
		std::cout << e.what() << std::endl;
		MessageBox(NULL, e.what(), NULL, NULL);
	}
	catch (cURLpp::LogicError &e)
	{
		std::cout << e.what() << std::endl;
		MessageBox(NULL, e.what(), NULL, NULL);
	}
	catch (...)
	{
		std::cout << "Undefined cURLpp Exception!!!" << std::endl;
		MessageBox(NULL, "Undefined cURLpp Exception!!!", NULL, NULL);
	}
}
