#pragma once

#include "StdAfx.h"

#include <stdlib.h>
#include <malloc.h>

#include <curlpp/cURLpp.hpp>
#include <curlpp/Exception.hpp>
#include <curlpp/Options.hpp>

#define MAX_FILE_LENGTH 16777215	// <- 24 bit length

class CConnection
{
public:
	// Public enumeration
	enum HTTP_Mode
	{
		HTTP_GET = 1,
		HTTP_POST = 2
	};

	// Helper Classes
	class DebugMemoryClass
	{
	public:
		// Helper Class for collecting Debug information
		DebugMemoryClass()
		{
			this->m_HWND = NULL;
			this->m_pBuffer = NULL;
			this->m_pBuffer = (char*) malloc(MAX_FILE_LENGTH * sizeof(char));
			this->m_Size = 0;
		};
		~DebugMemoryClass()
		{
			if (this->m_pBuffer)
			{
				free(this->m_pBuffer);
			}
		};
		void SetLoggingHWND(HWND hWnd)
		{
			this->m_HWND = hWnd;
		}
		static void* Realloc(void* ptr, size_t size)
		{
			if(ptr)
			{
				return realloc(ptr, size);
			}
			else
			{
				return malloc(size);
			}
		};
		// Callback must be declared static, otherwise it won't link...
		static int DebugMemoryCallback(cURL::CURL* pCURL, cURL::curl_infotype cURL_InfoType, char* pBuffer, size_t size, void* pData)
		{
			// Cast the void* data pointer to WriterMemoryClass...
			DebugMemoryClass* Memory = (DebugMemoryClass*) pData;

			switch (cURL_InfoType)
			{
			case cURL::CURLINFO_TEXT:
				if (Memory->m_pBuffer)
				{
					memcpy(&(Memory->m_pBuffer[Memory->m_Size]), pBuffer, size);
					Memory->m_Size += strlen(pBuffer);
					Memory->m_pBuffer[Memory->m_Size + 1] = 0;
				}

				SendMessage(Memory->m_HWND, WM_SETTEXT, (WPARAM) 0, (LPARAM) Memory->m_pBuffer);
			case cURL::CURLINFO_HEADER_OUT:
				if (Memory->m_pBuffer)
				{
					memcpy(&(Memory->m_pBuffer[Memory->m_Size]), "=> Send Header...\r\n", strlen("=> Send Header...\r\n"));
					Memory->m_Size += strlen("=> Send Header...\r\n");
					Memory->m_pBuffer[Memory->m_Size + 1] = 0;
				}

				SendMessage(Memory->m_HWND, WM_SETTEXT, (WPARAM) 0, (LPARAM) Memory->m_pBuffer);
				break;
			case cURL::CURLINFO_DATA_OUT:
				if (Memory->m_pBuffer)
				{
					memcpy(&(Memory->m_pBuffer[Memory->m_Size]), "=> Send data...\r\n", strlen("=> Send data...\r\n"));
					Memory->m_Size += strlen("=> Send data...\r\n");
					//strcat(Memory->m_pBuffer, "\r\n");
					//Memory->m_Size += 2;
					Memory->m_pBuffer[Memory->m_Size + 1] = 0;
				}

				SendMessage(Memory->m_HWND, WM_SETTEXT, (WPARAM) 0, (LPARAM) Memory->m_pBuffer);
				break;
			case cURL::CURLINFO_HEADER_IN:
				if (Memory->m_pBuffer)
				{
					memcpy(&(Memory->m_pBuffer[Memory->m_Size]), "<= Receive Header...\r\n", strlen("<= Receive Header...\r\n"));
					Memory->m_Size += strlen("<= Receive Header...\r\n");
					Memory->m_pBuffer[Memory->m_Size + 1] = 0;
				}

				SendMessage(Memory->m_HWND, WM_SETTEXT, (WPARAM) 0, (LPARAM) Memory->m_pBuffer);
				break;
			case cURL::CURLINFO_DATA_IN:
				if (Memory->m_pBuffer)
				{
					memcpy(&(Memory->m_pBuffer[Memory->m_Size]), "<= Receive Data...\r\n", strlen("<= Receive Data...\r\n"));
					Memory->m_Size += strlen("<= Receive Data...\r\n");
					Memory->m_pBuffer[Memory->m_Size + 1] = 0;
				}

				SendMessage(Memory->m_HWND, WM_SETTEXT, (WPARAM) 0, (LPARAM) Memory->m_pBuffer);
				break;
			case cURL::CURLINFO_SSL_DATA_IN:
				if (Memory->m_pBuffer)
				{
					memcpy(&(Memory->m_pBuffer[Memory->m_Size]), "<= Receive SSL Data...\r\n", strlen("<= Receive SSL Data...\r\n"));
					Memory->m_Size += strlen("<= Receive SSL Data...\r\n");
					Memory->m_pBuffer[Memory->m_Size + 1] = 0;
				}

				SendMessage(Memory->m_HWND, WM_SETTEXT, (WPARAM) 0, (LPARAM) Memory->m_pBuffer);
				break;
			case cURL::CURLINFO_SSL_DATA_OUT:
				if (Memory->m_pBuffer)
				{
					memcpy(&(Memory->m_pBuffer[Memory->m_Size]), "<= Send SSL Data...\r\n", strlen("<= Send SSL Data...\r\n"));
					Memory->m_Size += strlen("<= Send SSL Data...\r\n");
					Memory->m_pBuffer[Memory->m_Size + 1] = 0;
				}

				SendMessage(Memory->m_HWND, WM_SETTEXT, (WPARAM) 0, (LPARAM) Memory->m_pBuffer);
				break;
			default: /* in case a new one is introduced to shock us */
				return 0;
			}

			// Test if Buffer is initialized correctly & copy memory
			if (Memory->m_pBuffer)
			{
				memcpy(&(Memory->m_pBuffer[Memory->m_Size]), pBuffer, size);
				Memory->m_Size += strlen(pBuffer);
				Memory->m_pBuffer[Memory->m_Size + 1] = 0;
			}

			SendMessage(Memory->m_HWND, WM_SETTEXT, (WPARAM) 0, (LPARAM) Memory->m_pBuffer);

			return size;
		};

		// Public member vars

	protected:
		HWND m_HWND;
		char* m_pBuffer;
		size_t m_Size;

	private:
	};
	class WriterMemoryClass
	{
	public:
		// Helper Class for reading result from remote host
		WriterMemoryClass()
		{
			this->m_pBuffer = NULL;
			this->m_pBuffer = (char*) malloc(MAX_FILE_LENGTH * sizeof(char));
			this->m_Size = 0;
			this->m_pIStream = new std::istream(NULL);
		};
		~WriterMemoryClass()
		{
			if (this->m_pBuffer)
			{
				free(this->m_pBuffer);
			}
		};
		char* GetBuffer(void)
		{
			return (char*) this->m_pBuffer;	
		};
		size_t GetSize(void)
		{
			return this->m_Size;
		};
		static void* Realloc(void* ptr, size_t size)
		{
			if(ptr)
			{
				return realloc(ptr, size);
			}
			else
			{
				return malloc(size);
			}
		};
		// Callback must be declared static, otherwise it won't link...
		// When invoking HTTP POST requests data pointer seems to be broken... Don't know why this is so.
		static size_t WriteMemoryCallback(char* ptr, size_t size, size_t nmemb, void* data)
		{
			// Calculate the original size of the incoming buffer
			size_t realsize = size * nmemb;

			// Cast the void* data pointer to WriterMemoryClass...
			WriterMemoryClass* Memory = (WriterMemoryClass*) data;

			// Test if Buffer is initialized correctly & copy memory
			if (Memory->m_pBuffer)
			{
				memcpy(&(Memory->m_pBuffer[Memory->m_Size]), ptr, realsize);

				Memory->m_Size += realsize;

				Memory->m_pBuffer[Memory->m_Size + 1] = 0;
			}

			return realsize;
		};

		// Public member vars

	protected:
		std::istream* m_pIStream;
		char* m_pBuffer;
		size_t m_Size;

	private:
	};
	// Helper Class for writing result to remote host
	class ReaderMemoryClass
	{
	public:
		ReaderMemoryClass()
		{
			this->m_pBuffer = NULL;
			this->m_pBuffer = (char*) malloc(MAX_FILE_LENGTH * sizeof(char));
			this->m_Size = 0;
		};
		~ReaderMemoryClass()
		{
			if (this->m_pBuffer)
			{
				free(this->m_pBuffer);
			}
		};
		char* GetBuffer(void)
		{
			return this->m_pBuffer;	
		};
		size_t GetSize(void)
		{
			return this->m_Size;
		};
		void SetBuffer(char* pBuffer)
		{
			strcpy(this->m_pBuffer, pBuffer);
			this->m_pBuffer[strlen(pBuffer) + 1] = 0;
		};
		static void* Realloc(void* ptr, size_t size)
		{
			if(ptr)
			{
				return realloc(ptr, size);
			}
			else
			{
				return malloc(size);
			}
		};
		// Callback must be declared static, otherwise it won't link...
		static size_t ReadMemoryCallback(char* buffer, size_t size, size_t nitems, void* instream)
		{
			ReaderMemoryClass* Memory = (ReaderMemoryClass*) instream;

			Memory->m_Size = size * nitems;
			strncpy(buffer, Memory->m_pBuffer, Memory->m_Size);

			return size * nitems;
		};

		// Public member vars

	protected:
		char* m_pBuffer;
		size_t m_Size;

	private:
	};


	// Constructors & Destructors
	CConnection(HTTP_Mode Mode = HTTP_GET, char* pHost = "localhost", char* pFilePath = "", char* pArg = "", unsigned int Port = 80, bool binary = false);
	~CConnection(void);

	// Setter Methods
	void SetHostAdress(char* pHost);
	void SetFilePath(char* pPath);
	void SetArg(char* pArg);
	void SetPortNumber(unsigned int PortNumber);
	void SetLoggingHWND(HWND hWnd);

	// Getter Methods
	char* GetHostAdress(void);
	char* GetFilePath(void);
	char* GetArg(void);
	unsigned int GetPortNumber(void);
	char* GetResult(void);
	int GetResultLength(void);

	// Helper Methods
	void PerformAction(void);

	// Public vars
	CConnection::WriterMemoryClass* m_pWriterChunk;		// <- Here the result is written
	CConnection::ReaderMemoryClass* m_pReaderChunk;		// <- Here the data to send is stored
	CConnection::DebugMemoryClass*	m_pDebugChunk;		// <- Hete the debug data is stored

protected:
	// Protected Methods
	void Init(char* pHost = "localhost", char* pFilePath = "", char* pArg = "", unsigned int Port = 80, HTTP_Mode Mode = HTTP_GET);

	// Protected member vars


private:
	// Private Methods


	// Private member vars
	char* m_pHostAdress;
	char* m_pFilePath;
	char* m_pArg;
	unsigned int m_PortNumber;
	cURLpp::Easy* m_pRequest;
};
