#ifndef CONTROLLER_INCLUDE
#define CONTROLLER_INCLUDE 1

#pragma once
#include <new.h>
#include "SOAPClient.h"

#include <fstream>

class SimpleControl
{
public:
    SimpleControl (HWND hwndParent, int id, BOOL initialState=TRUE)
        : _hWnd (GetDlgItem (hwndParent, id))
    {
        if (initialState == FALSE)
         Disable();
   }
   void Hide ()
   {
      ::ShowWindow(_hWnd , SW_HIDE);
   }
   void Show ()
   {
      ::ShowWindow(_hWnd , SW_SHOW);
   }
   BOOL IsVisible()
   {
      return(::IsWindowVisible(_hWnd ));
   }
   void SetFocus ()
   {
       ::SetFocus (_hWnd);
   }
   void Enable()
   {
       ::EnableWindow(_hWnd , TRUE);
   }
   void Disable()
   {
       ::EnableWindow(_hWnd , FALSE);
   }
	HWND Hwnd () const { return _hWnd; }

protected:
//public:
    HWND _hWnd;
};

class Button : public SimpleControl
{
public:
   Button(HWND hwndParent, int id, BOOL initialState=TRUE)
       : SimpleControl(hwndParent, id, initialState)
   {}
   void SetName( char const * newName )
   {
      SendMessage(_hWnd, WM_SETTEXT, 0, (LPARAM)newName );
   }
};

class CheckBox : public Button
{
public:
   CheckBox (HWND hwndParent, int id, BOOL initialState=TRUE)
       : Button(hwndParent, id, initialState)
   {}
   BOOL IsChecked()
   {
      return( SendMessage(_hWnd, BM_GETCHECK, 0, 0) == BST_CHECKED );
   }
   void Check()
   {
      SendMessage(_hWnd, BM_SETCHECK, (WPARAM) BST_CHECKED, 0);
   }
   void UnCheck()
   {
      SendMessage(_hWnd, BM_SETCHECK, (WPARAM) BST_UNCHECKED, 0);
   }
};

class RadioButton : public Button
{
public:
   RadioButton (HWND hwndParent, int id, BOOL initialState=TRUE)
       : Button(hwndParent, id, initialState)
   {}
   BOOL IsSelected()
   {
      return( SendMessage(_hWnd, BM_GETCHECK, 0, 0) == BST_CHECKED );
   }
   void Select()
   {
      SendMessage(_hWnd, BM_SETCHECK, (WPARAM) BST_CHECKED, 0);
   }
};

class RichEdit : public SimpleControl
{
public:
	RichEdit (HWND hwndParent, int id, BOOL initialState=TRUE)
		:	SimpleControl (hwndParent, id, initialState)
	{
		SendMessage(_hWnd, EM_SETTEXTMODE, (WPARAM) TM_PLAINTEXT, (LPARAM) 0);
	}

	void SetSel(CHARRANGE charRange)
	{
		// If the cpMin and cpMax members are equal, the range is empty.
		// The range includes everything if cpMin is 0 and cpMax is 1.
		SendMessage(_hWnd, EM_EXSETSEL, (WPARAM) 0, (LPARAM) &charRange);
	}

	void GetSelText(char* buf)
	{
		SendMessage(_hWnd, EM_GETSELTEXT, (WPARAM) 0, (LPARAM) buf);
	}

	int GetTextLength(GETTEXTLENGTHEX texLength)
	{
		return (int) SendMessage(_hWnd, EM_GETTEXTLENGTHEX, (WPARAM) &texLength, (LPARAM) 0);
	}

	void SetText(char* buf)
	{
		// If codepage is 1200 (Unicode code page), no translation is done.
		// If codepage is CP_ACP, the system code page is used.
		SETTEXTEX settextex;
		settextex.flags = ST_DEFAULT;
		settextex.codepage = CP_ACP;

		SendMessage(_hWnd, EM_SETTEXTEX, (WPARAM) &settextex, (LPARAM) buf);
	}
};

class Edit : public SimpleControl
{
public:
   Edit (HWND hwndParent, int id, BOOL initialState=TRUE)
        :	SimpleControl (hwndParent, id, initialState)
   {}

   void SetString (char* buf)
   {
       SendMessage (_hWnd, WM_SETTEXT, 0, (LPARAM) buf);
   }

	void AddString(char* buf)
	{
		char* tmpBuf;
		char* finalStr;
		int strLen = 0;

		strLen = SendMessage(_hWnd, WM_GETTEXTLENGTH, 0, 0);
		tmpBuf = (char*) malloc((strLen/* + 1*/) * sizeof(char));
		SendMessage (_hWnd, WM_GETTEXT, (WPARAM) strLen + 1, (LPARAM) tmpBuf);		// strLen + 1 because of null terminating string
		finalStr = (char*) malloc(strlen(tmpBuf) + strlen(buf) + 1 * sizeof(char));
		sprintf(finalStr, tmpBuf, buf);

		this->SetString(finalStr);
	}

   // code is the HIWORD (wParam)
   static BOOL IsChanged (int code)
   {
       return code == EN_CHANGE;
   }

   int GetLength ()
   {
       return (int)(SendMessage (_hWnd, WM_GETTEXTLENGTH, 0, 0));
   }

   void GetString (char* buf, int len)
   {
       SendMessage (_hWnd, WM_GETTEXT, (WPARAM) len, (LPARAM) buf);
   }

   void Select ()
   {
       SendMessage (_hWnd, EM_SETSEL, 0, -1);
   }

   void ClearSelection ()
   {
       SendMessage (_hWnd, EM_SETSEL, -1, 0);
   }
};

class Tree : public SimpleControl
{
public:
   Tree (HWND hwndParent, int id, HINSTANCE hInst=NULL, BOOL initialState=TRUE)
        :	SimpleControl (hwndParent, id, initialState)
   {
	   if (hInst)
	   {
//			HIMAGELIST hImageList = ImageList_Create (16, 16, ILC_COLOR16, 2, 10);
//			HBITMAP hBitMap = LoadBitmap (hInst, MAKEINTRESOURCE (IDB_TREE) );
//			ImageList_Add (hImageList, hBitMap, NULL);
//			DeleteObject (hBitMap);
//			SendMessage (_hWnd, TVM_SETIMAGELIST, (WPARAM) 0, (LPARAM) hImageList);
	   }
   }

   HTREEITEM AddItem(TV_INSERTSTRUCT* insert)
   {
		return (HTREEITEM)SendMessage (_hWnd, TVM_INSERTITEM, 0, (LPARAM) insert);
   }

   HTREEITEM GetParentItem()
   {
		return (HTREEITEM)SendMessage (_hWnd, TVM_SELECTITEM, (WPARAM) TVGN_PARENT, (LPARAM) 0);
   }

   BOOL DeleteItem(TV_ITEM* item)
   {
		return TreeView_DeleteItem(_hWnd, item);
   }

   BOOL Reset()
   {
		return (BOOL)SendMessage (_hWnd, TVM_DELETEITEM, (WPARAM) 0, (LPARAM) TVI_ROOT);
   }
};

class Slider : public SimpleControl
{
public:
   Slider (HWND hwndParent, int id, BOOL initialState=TRUE)
        :	SimpleControl (hwndParent, id, initialState)
   {}

	void SetRange(int range)
	{
		SendMessage (_hWnd, TBM_SETRANGE, 0, MAKELONG(0, range));
	}

   void SetPosition(int pos)
   {
		SendMessage (_hWnd, TBM_SETPOS, (WPARAM) TRUE, (LPARAM) pos);
   }

   int GetPosition(int pos)
   {
		return SendMessage (_hWnd, TBM_GETPOS, 0, 0);
   }
};

class List : public SimpleControl
{
public:
	List (HWND hwndParent, int id, BOOL initialState=TRUE)
        :	SimpleControl (hwndParent, id, initialState)
	{}

	void InsertString(char* buf, int ind = -1)
	{
		SendMessage (_hWnd, LB_INSERTSTRING, (WPARAM) ind, (LPARAM) buf);
	}

	int SelectString(char* buf, int ind = -1)
	{
		return (int)(SendMessage (_hWnd, LB_SELECTSTRING, (WPARAM) ind, (LPARAM) buf));
	}

	void AddString(char* buf)
	{
		SendMessage (_hWnd, LB_ADDSTRING, 0, (LPARAM) buf);
	}

	void DeleteString(int ind)
	{
		SendMessage (_hWnd, LB_DELETESTRING, (WPARAM) ind, 0);
	}

	int GetCount()
	{
		return (int) (SendMessage (_hWnd, LB_GETCOUNT, 0, 0));
	}

	int GetCurSel()
	{
		return (int) (SendMessage (_hWnd, LB_GETCURSEL, 0, 0));
	}
};

class CController
{
public:
	CController(void);
	CController(HWND hwnd);
	CController(HWND hwnd, HINSTANCE hInst);
	~CController(void);

	void Command (HWND hwnd, int controlID, int command);
private:
    Edit        _edit;
	Tree		_tree;
	Slider		_slider;
	Edit		_editEndpoint;
	Edit		_editMethod;
	Edit		_editService;
	Edit		_editModus;
	Edit		_editLogging;
	char m_szFileName[MAX_PATH];
	char* m_szFileText;
	char* m_szResultText;
};

#endif // CONTROLLER_INCLUDE